{% extends "admin/change_list.html" %}
{% load i18n admin_urls %}

{% block object-tools-items %}
{{ block.super }}
  <li>
    <a href="/admin/subscription/upload/">
      Upload CSV of messages
    </a>
  </li>
{% endblock %}
        
