{% extends "admin/base_site.html" %}
{% load i18n admin_static admin_list %}
{% load admin_urls %}

{% if not is_popup %}
    {% block breadcrumbs %}
        <div class="breadcrumbs">
            <a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
             &rsaquo; <a href="/admin/subscription/">Subscriptions</a>
             &rsaquo; Upload
        </div>
    {% endblock %}
{% endif %}

{% block content %}

    <div id="content">
        <div id="content-main">
            <div class="module" id="changelist">
                <div class="results">
                    <h2>Upload CSV</h2>
                    <form id="changelist-form" method="post" method="POST" action="." enctype="multipart/form-data">
                        {% csrf_token %}
                        {{ form.as_p}}
                        <input type="submit" class="default" value="Upload" />
                    </form>
                </div>
            </div>
        </div>
    </div>

{% endblock %}
